@@ -30,12 +30,16 @@ pub mod dp {
     }
     /// Finds subsets sum of a target value. It can accept negative values.
     ///
+    /// # Arguments
+    /// * `arr` - An array.
+    /// * `value` - The value to the sum of the subset comes.
+    /// * `max_length` - The maximum length of combinations of the answer.
     /// # Example
     /// ```
     ///
     /// use dpss::dp::find_subset;
     /// let arr = vec![-1, -3, -2, 6, 12, 48];
-    /// let result = find_subset(&arr, 0);
+    /// let result = find_subset(&arr, 0, 4);
     /// let route1: Vec<i32> = vec![6, -2, -3, -1];
     /// let answer: Vec<Vec<i32>> = vec![route1];
     /// assert_eq!(result, answer);
@@ -45,11 +49,11 @@ pub mod dp {
     /// ```
     ///
     /// use dpss::dp::find_subset;
-    /// let result = find_subset(&vec![1, 2, 3, -4, 5], 1);
+    /// let result = find_subset(&vec![1, 2, 3, -4, 5], 1, 2);
     /// println!("{:?}", result);
     /// ```
     /// output: `[[1], [4, -3]]`
-    pub fn find_subset(arr: &Vec<i32>, value: i32) -> Vec<Vec<i32>> {
+    pub fn find_subset(arr: &Vec<i32>, value: i32, max_length: usize) -> Vec<Vec<i32>> {
         use std::cmp::max;
         use std::cmp::min;
         // https://stackoverflow.com/questions/43078142/subset-sum-with-negative-values-in-c-or-c
@@ -60,7 +64,7 @@ pub mod dp {
             for i in arr {
                 b.push(*i as u32);
             }
-            let result = find_subset_fast_only_positive(&b, value as usize);
+            let result = find_subset_fast_only_positive(&b, value as usize, max_length);
             for i in result {
                 let mut tempvec = Vec::with_capacity(i.len());
                 for j in i {
@@ -80,7 +84,7 @@ pub mod dp {
             // If we find the sum is the same as the target, we will return the result.
             for i in 1..arr.len() + 1 {
                 let result =
-                    find_subset_fast_only_positive(&b, (value + i as i32 * offset as i32) as usize);
+                    find_subset_fast_only_positive(&b, (value + i as i32 * offset as i32) as usize, max_length);
                 for res in result {
                     let mut tempsum: i32 = 0;
                     let mut new_res: Vec<i32> = Vec::with_capacity(res.len());
@@ -105,6 +109,7 @@ pub mod dp {
         route: &mut Vec<u32>,
         answer: &mut Vec<Vec<u32>>,
         a_min: &u32,
+        max_length: usize,
     ) {
         // This code is mostly copied from https://drken1215.hatenablog.com/entry/2019/12/17/190300
         // We follow the dp table backward to find combinations of subsets.
@@ -112,19 +117,25 @@ pub mod dp {
         if i == 0 {
             if j == 0 {
                 // Only if we reach the root of the dp table, we choose the combination as an answer.
-                answer.push(route.clone());
+                if route.len() <= max_length {
+                    answer.push(route.clone());
+                }
             }
             return;
         }
 
+        if route.len() > max_length {
+            return;
+        }
+
         if dp[i - 1][j] != 0 {
-            rec(dp, arr, i - 1, j, route, answer, a_min);
+            rec(dp, arr, i - 1, j, route, answer, a_min, max_length);
         }
 
         if j as i32 - arr[i - 1] as i32 >= 0 && dp[i - 1][j - arr[i - 1] as usize] != 0 {
             // Choose this element as arr candidate for an answer.
             route.push(arr[i - 1]);
-            rec(dp, arr, i - 1, j - arr[i - 1] as usize, route, answer, a_min);
+            rec(dp, arr, i - 1, j - arr[i - 1] as usize, route, answer, a_min, max_length);
             // Remove this element after we reach i == 0 regardless of whether we reach j == 0.
             route.pop();
         }
@@ -182,12 +193,15 @@ pub mod dp {
         assert_eq!(result, answer);
     }
     /// Finds subsets sum of a target value. It can't accept negative values but relatively faster.
-    ///
+    /// # Arguments
+    /// * `arr` - An array.
+    /// * `value` - The value to the sum of the subset comes.
+    /// * `max_length` - The maximum length of combinations of the answer.
     /// # Example
     /// ```
     ///
     /// use dpss::dp::find_subset_fast_only_positive;
-    /// let result = find_subset_fast_only_positive(&vec![1, 2, 3], 3);
+    /// let result = find_subset_fast_only_positive(&vec![1, 2, 3], 3, 2);
     /// let route1: Vec<u32> = vec![2, 1];
     /// let route2: Vec<u32> = vec![3];
     /// let answer: Vec<Vec<u32>> = vec![route1, route2];
@@ -197,11 +211,11 @@ pub mod dp {
     /// ```
     ///
     /// use dpss::dp::find_subset_fast_only_positive;
-    /// let result = find_subset_fast_only_positive(&vec![1, 2, 3, 4, 5], 10);
+    /// let result = find_subset_fast_only_positive(&vec![1, 2, 3, 4, 5], 10, 4);
     /// println!("{:?}", result);
     /// ```
     /// output: `[[5, 3, 2], [5, 4, 1], [4, 3, 2, 1]]`
-    pub fn find_subset_fast_only_positive(arr: &Vec<u32>, value: usize) -> Vec<Vec<u32>> {
+    pub fn find_subset_fast_only_positive(arr: &Vec<u32>, value: usize, max_length: usize) -> Vec<Vec<u32>> {
         // dp is a table that stores the information of subset sum.
         // dp[i][j] is the number of ways to make sum j with i element.
         // We follow from the start of this table.
@@ -227,7 +241,7 @@ pub mod dp {
         let mut route: Vec<u32> = Vec::with_capacity(a_length);
         let mut answer: Vec<Vec<u32>> = Vec::with_capacity(a_length);
 
-        rec(&dp, &arr, a_length, value, &mut route, &mut answer, &a_min);
+        rec(&dp, &arr, a_length, value, &mut route, &mut answer, &a_min, max_length);
         answer
     }
 
@@ -239,11 +253,15 @@ pub mod dp {
     /// Finds the integers from two vectors that sum to the same value.
     /// This method assumes that the two vectors have One-to-Many relationships.
     /// Each integer of the `key` vector corresponds to the multiple integers of the `value` vector.
+    /// # Arguments
+    /// * `key` - An array.
+    /// * `targets` - An array.
+    /// * `max_length` - The maximum length of combinations of the answer.
     /// # Example
     /// ```
     ///
     /// use dpss::dp::sequence_matcher;
-    /// let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3]);
+    /// let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3], 4);
     /// assert_eq!(answer, vec![
     ///     vec![
     ///         (3, vec![3]),
@@ -263,16 +281,17 @@ pub mod dp {
     /// ]);
     ///
     /// let answer_unchanged: Vec<Vec<(i32, Vec<i32>)>> = Vec::new();
-    /// let answer = sequence_matcher(&mut vec![10, 20], &mut vec![9, 21]);
+    /// let answer = sequence_matcher(&mut vec![10, 20], &mut vec![9, 21], 1);
     /// assert_eq!(answer, answer_unchanged);
     /// ```
     pub fn sequence_matcher(
         key: &mut Vec<i32>,
         targets: &mut Vec<i32>,
+        max_target_length: usize,
     ) -> Vec<Vec<(i32, Vec<i32>, )>> {
         let mut group: Vec<(i32, Vec<i32>)> = Vec::with_capacity(targets.len());
         let mut answer: Vec<Vec<(i32, Vec<i32>)>> = Vec::with_capacity(targets.len());
-        sequence_matcher_core(key, targets, &mut group, &mut answer);
+        sequence_matcher_core(key, targets, &mut group, &mut answer, max_target_length);
         if answer.len() == 0 {
             println!("Can't find any combination.");
         }
@@ -284,6 +303,7 @@ pub mod dp {
         targets: &mut Vec<i32>,
         group: &mut Vec<(i32, Vec<i32>)>,
         answer: &mut Vec<Vec<(i32, Vec<i32>)>>,
+        max_target_length: usize
     ) {
         if key.len() == 0 && targets.len() == 0 {
             answer.push(group.clone());
@@ -296,7 +316,7 @@ pub mod dp {
             return;
         }
 
-        let set_: Vec<Vec<i32>> = find_subset(&targets, key[0]);
+        let set_: Vec<Vec<i32>> = find_subset(&targets, key[0], max_target_length);
         for set in set_ {
             group.push((key[0], set.clone()));
             let i2 = key[0].clone();
@@ -305,7 +325,7 @@ pub mod dp {
             }
             vec_remove(key, key[0]);
 
-            sequence_matcher_core(key, targets, group, answer);
+            sequence_matcher_core(key, targets, group, answer, max_target_length);
             group.pop();
             for el in set.clone() {
                 targets.push(el);
@@ -317,16 +337,22 @@ pub mod dp {
     /// Finds the integers from two vectors that sum to the same value.
     /// This method assumes that the two vectors have Many-to-Many relationships.
     /// Each integer of the `keys` vector corresponds to the multiple integers of the `targets` vector.
-    /// With this method, we can find multiple combinations of the integers.
+    /// With this method, we can find some combinations of the integers.
     /// `n_candidates` is the number of candidates to be selected.
     /// `max_key_length` is the maximum length of the keys as a group.
-    /// Especially in long sequences, this method is very slow so `n_candidates` and `max_key_length` should be small.
+    /// Especially in long sequences, this method is slow so `n_candidates`, `max_key_length`, and `max_target_length` should be small.
+    /// # Arguments
+    /// * `keys` - An array.
+    /// * `targets` - An array.
+    /// * `n_candidates` - The maximum length of combinations of the answer.
+    /// * `max_key_length` - The maximum length of combinations of the keys.
+    /// * `max_target_length` - The maximum length of combinations of the targets.
     /// # Example
     ///
     /// ```rust
     ///
     ///use dpss::dp::sequence_matcher_m2m;
-    ///let answer = sequence_matcher_m2m(&mut vec![1980, 2980, 3500, 4000, 1050], &mut vec![1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 10, 5);
+    ///let answer = sequence_matcher_m2m(&mut vec![1980, 2980, 3500, 4000, 1050], &mut vec![1950, 2900, 30, 80, 3300, 200, 3980, 1050, 20], 10, 5, 10);
     ///assert_eq!(answer[0], vec![
     ///    (vec![1050],
     ///     vec![1050]),
@@ -368,6 +394,7 @@ pub mod dp {
         targets: &mut Vec<i32>,
         n_candidates: usize,
         max_key_length: usize,
+        max_target_length: usize,
     ) -> Vec<Vec<(Vec<i32>, Vec<i32>)>> {
         use rand::seq::SliceRandom;
 
@@ -379,7 +406,7 @@ pub mod dp {
             return answer;
         }
         for _i in 0..n_candidates {
-            sequence_matcher_core_m2m(keys, targets, &mut group, &mut answer, 1, max_key_length);
+            sequence_matcher_core_m2m(keys, targets, &mut group, &mut answer, 1, max_key_length, max_target_length);
             keys.shuffle(&mut rng);
         }
         answer.sort();
@@ -397,6 +424,7 @@ pub mod dp {
         answer: &mut Vec<Vec<(Vec<i32>, Vec<i32>)>>,
         n_key: usize,
         max_key_length: usize,
+        max_target_length: usize,
     ) {
         if keys.iter().sum::<i32>() != targets.iter().sum() {
             return;
@@ -423,9 +451,9 @@ pub mod dp {
         if targets.iter().max().unwrap() == &0{
             return;
         }
-        let set_: Vec<Vec<i32>> = find_subset(&targets, sum_key);
+        let set_: Vec<Vec<i32>> = find_subset(&targets, sum_key, max_target_length);
         if set_.len() == 0 {
-            sequence_matcher_core_m2m(keys, targets, group, answer, n_key + 1, max_key_length);
+            sequence_matcher_core_m2m(keys, targets, group, answer, n_key + 1, max_key_length, max_target_length);
         }
         for set in set_ {
             let mut _set = Vec::from(set.clone());
@@ -439,7 +467,7 @@ pub mod dp {
             for i in vec_key.clone() {
                 vec_remove(keys, i);
             }
-            sequence_matcher_core_m2m(keys, targets, group, answer, n_key, max_key_length);
+            sequence_matcher_core_m2m(keys, targets, group, answer, n_key, max_key_length, max_target_length);
             group.pop();
             for el in set.clone() {
                 targets.push(el);
@@ -452,18 +480,18 @@ pub mod dp {
 
     #[test]
     fn test_sequence_matcher() {
-        let answer = sequence_matcher(&mut vec![1, 6], &mut vec![1, 3, 3]);
+        let answer = sequence_matcher(&mut vec![1, 6], &mut vec![1, 3, 3], 2);
         assert_eq!(answer, vec![vec![(1, vec![1]), (6, vec![3, 3])]]);
 
-        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![-10, 20, 16, 4]);
+        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![-10, 20, 16, 4], 2);
         assert_eq!(answer, vec![vec![(10, vec![20, -10]), (20, vec![4, 16]),],]);
 
         let answer_unchanged: Vec<Vec<(i32, Vec<i32>)>> = Vec::new();
 
-        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![10, 21]);
+        let answer = sequence_matcher(&mut vec![10, 20], &mut vec![10, 21], 2);
         assert_eq!(answer, answer_unchanged);
 
-        let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3]);
+        let answer = sequence_matcher(&mut vec![3, 5, 7], &mut vec![1, 5, -3, 4, 5, 3], 4);
         assert_eq!(
             answer,
             vec![
@@ -480,7 +508,7 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![6, 7, 3, 2, -9, -3, 8, 3, 6, -10],
             &mut vec![3, 2, -6, -8, 2, -9, 0, -5, -3, 37],
-            10, 10
+            10, 10, 10
         );
         assert_eq!(
             answer[0],
@@ -494,7 +522,7 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![6, 7, 3, 2, -9],
             &mut vec![-3, 8, 3, 6, -5],
-            10, 4
+            10, 4, 10
         );
         assert_eq!(
             answer[0],
@@ -507,7 +535,7 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![9, 0, 1, 7, 1],
             &mut vec![7, 2, 8, 0, 1],
-            10, 3
+            10, 3, 10
         );
         assert_eq!(
             answer[0],
@@ -519,7 +547,7 @@ pub mod dp {
         );
 
         let answer =
-            sequence_matcher_m2m(&mut vec![1, 2, 3, 4, 5], &mut vec![11, -8, 14, -7, 5], 10, 4);
+            sequence_matcher_m2m(&mut vec![1, 2, 3, 4, 5], &mut vec![11, -8, 14, -7, 5], 10, 4, 10);
         assert_eq!(
             answer[0],
             vec![(vec![1], vec![-8, -7, 5, 11]), (vec![2, 3, 4, 5], vec![14]),]
@@ -528,7 +556,7 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![1000, 1100, 150, 123, 5, 10],
             &mut vec![2100, 273, 4, 11],
-            10, 4
+            10, 4, 10
         );
         assert_eq!(
             answer[0],
@@ -549,17 +577,17 @@ pub mod dp {
         let answer = sequence_matcher_m2m(
             &mut vec![1000, 1100, 150, 123, 5, 10],
             &mut vec![1000, 1200],
-            10, 6
+            10, 6, 10
         );
         assert_eq!(answer.len(), 0);
 
-        let answer = sequence_matcher_m2m(&mut vec![-950, 10000], &mut vec![5000, 4000, 50], 10, 2);
+        let answer = sequence_matcher_m2m(&mut vec![-950, 10000], &mut vec![5000, 4000, 50], 10, 2, 10);
         assert_eq!(answer[0], vec![(vec![-950, 10000], vec![50, 4000, 5000]),]);
 
         let answer = sequence_matcher_m2m(
             &mut vec![99, 68, -74, 72, -38, 22],
             &mut vec![36, -23, -92, 88, 67, 73],
-            10, 5
+            10, 5, 10
         );
         assert_eq!(
             answer[0],
@@ -569,7 +597,7 @@ pub mod dp {
             ]
         );
 
-        let answer = sequence_matcher_m2m(&mut vec![1, 2, 3, 4], &mut vec![1, 5], 10, 2);
+        let answer = sequence_matcher_m2m(&mut vec![1, 2, 3, 4], &mut vec![1, 5], 10, 2, 10);
 
         assert_eq!(answer.len(), 0);
     }
@@ -582,19 +610,19 @@ mod tests {
 
     #[test]
     fn test_find_subset_fast_only_positive() {
-        let result = dp::find_subset_fast_only_positive(&vec![1, 2, 3], 3);
+        let result = dp::find_subset_fast_only_positive(&vec![1, 2, 3], 3, 2);
         let route1: Vec<u32> = vec![2, 1];
         let route2: Vec<u32> = vec![3];
         let answer: Vec<Vec<u32>> = vec![route1, route2];
         assert_eq!(result, answer);
 
-        let result = dp::find_subset_fast_only_positive(&vec![0, 3, 5, 10], 3);
+        let result = dp::find_subset_fast_only_positive(&vec![0, 3, 5, 10], 3, 2);
         let route1: Vec<u32> = vec![3];
         let route2: Vec<u32> = vec![3, 0];
         let answer: Vec<Vec<u32>> = vec![route1, route2];
         assert_eq!(result, answer);
 
-        let result = dp::find_subset_fast_only_positive(&vec![1, 2, 3, 0], 3);
+        let result = dp::find_subset_fast_only_positive(&vec![1, 2, 3, 0], 3, 3);
         let route1: Vec<u32> = vec![2, 1];
         let route2: Vec<u32> = vec![3];
         let route3: Vec<u32> = vec![0, 2, 1];
@@ -605,27 +633,33 @@ mod tests {
 
     #[test]
     fn test_find_subset() {
-        let result = dp::find_subset(&vec![1, 2, 3], 3);
+        let result = dp::find_subset(&vec![1, 2, 3], 3, 2);
         let route1: Vec<i32> = vec![2, 1];
         let route2: Vec<i32> = vec![3];
         let answer: Vec<Vec<i32>> = vec![route1, route2];
         assert_eq!(result, answer);
 
-        let result = dp::find_subset(&vec![1, 2, 3, 4, 5], 10);
+        let result = dp::find_subset(&vec![1, 2, 3, 4, 5], 10, 4);
         let route1: Vec<i32> = vec![4, 3, 2, 1];
         let route2: Vec<i32> = vec![5, 3, 2];
         let route3: Vec<i32> = vec![5, 4, 1];
         let answer: Vec<Vec<i32>> = vec![route1, route2, route3];
         assert_eq!(result, answer);
 
+        let result = dp::find_subset(&vec![1, 2, 3, 4, 5], 10, 3);
+        let route2: Vec<i32> = vec![5, 3, 2];
+        let route3: Vec<i32> = vec![5, 4, 1];
+        let answer: Vec<Vec<i32>> = vec![route2, route3];
+        assert_eq!(result, answer);
+
         let arr = vec![75, 467, 512, -835, 770, -69, 10];
-        let result = dp::find_subset(&arr, 711);
+        let result = dp::find_subset(&arr, 711, 3);
         let route1: Vec<i32> = vec![10, -69, 770];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
 
         let arr = vec![-3, 10, 56, -33, 65, -9, 8, 72, 63, 35];
-        let result = dp::find_subset(&arr, 7);
+        let result = dp::find_subset(&arr, 7, 4);
         let route1: Vec<i32> = vec![10, -3];
         let route2: Vec<i32> = vec![35, 8, -33, -3];
         let answer: Vec<Vec<i32>> = vec![route1, route2];
@@ -635,31 +669,31 @@ mod tests {
             73209, 95597, 84735, 40496, 83553, 95595, -628, 201, 27597, 7904, 98445, 6241, 33002,
             -776, -711, 45552, 86746, 84248, 66278, 37475,
         ];
-        let result = dp::find_subset(&arr, 72782);
+        let result = dp::find_subset(&arr, 72782, 3);
         let route1: Vec<i32> = vec![201, -628, 73209];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
 
         let arr = vec![-1, 2, 3];
-        let result = dp::find_subset(&arr, -1);
+        let result = dp::find_subset(&arr, -1, 1);
         let route1: Vec<i32> = vec![-1];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
 
         let arr = vec![-10, 5, -2];
-        let result = dp::find_subset(&arr, -5);
+        let result = dp::find_subset(&arr, -5, 2);
         let route1: Vec<i32> = vec![5, -10];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
 
         let arr = vec![-3, -5, -7];
-        let result = dp::find_subset(&arr, -15);
+        let result = dp::find_subset(&arr, -15, 3);
         let route1: Vec<i32> = vec![-7, -5, -3];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
 
         let arr = vec![-100, 10, 20];
-        let result = dp::find_subset(&arr, -70);
+        let result = dp::find_subset(&arr, -70, 3);
         let route1: Vec<i32> = vec![20, 10, -100];
         let answer: Vec<Vec<i32>> = vec![route1];
         assert_eq!(result, answer);
